import re
import textwrap

from . import markdownparser
from . import settings


class Comment(object):
    newlinepattern = re.compile(r'\n|\r|\r\n')
    def __init__(self, comment):
        self.comment = self._clean(comment)
        self.unparsed_comment = comment

    def _clean(self, comment):
        whitespacenormalized = textwrap.dedent(comment).rstrip()
        if len(whitespacenormalized) > 1 and whitespacenormalized[0] == '*':
            lines = self.newlinepattern.split(whitespacenormalized)
            starlines = [line[1:] for line in lines if line.startswith('*')]
            if len(starlines) == len(lines):
                return textwrap.dedent('\n'.join(starlines))
        else:
            return whitespacenormalized

    def __str__(self):
        return str(unicode(self).encode('ascii', 'ignore'))

    def __unicode__(self):
        return self.comment

    def to_html(self, settingsobject=None):
        settingsobject = settingsobject or settings.Settings()
        return markdownparser.to_html(self.comment, settingsobject)


class Parse(object):
    """
    Parses the given ``text`` looking for comments matching one of the following styles::

        /**
        A comment.
        */
       
       /**
        * A comment.
        */
       
    This does NOT work::

        /** A comment */
    """

    commentpattern = re.compile(r'(?<=/\*\*)[ \t]*(?:\n|\r|\r\n)(.*?)(?=\*/)', re.DOTALL)

    def __init__(self, text):
        """
        Parameters:
            text (unicode): The text to parse.
        """
        self.text = text
        self.commentbodies = self.commentpattern.findall(self.text)

    def __getitem__(self, index):
        return Comment(self.commentbodies[index])

    def __len__(self):
        return len(self.commentbodies)

    def __iter__(self):
        for commentbody in self.commentbodies:
            yield Comment(commentbody)

    def unicodelist(self):
        return map(lambda c: unicode(c), self)


import re
import markdown
import os
from markdown.extensions import Extension
from markdown.extensions import codehilite


class StyleExampleExtension(Extension):
    def __init__(self, configs):
        # define default configs
        self.config = {
            'settings': [None, "vitalstyles.settings.Settings object."],
        }

        # Set config
        self.setConfig('settings', configs['settings'])


    def extendMarkdown(self, md, md_globals):
        md.registerExtension(self)
        md.preprocessors.add('vitalstyles_example_block',
            StyleExampleBlockPreprocessor(md, settingsobject=self.config['settings'][0]),
            ">normalize_whitespace")



class StyleExampleBlockPreprocessor(markdown.preprocessors.Preprocessor):
    FENCED_BLOCK_RE = re.compile(r'''
(?P<fence>^\${3,})[ ]*         # Opening $$$
(?:(?P<examplestyle>isolated|fullwidth)?[ ]*\n)
(?P<html>.*?)(?<=\n)
(?P=fence)[ ]*$''', re.MULTILINE | re.DOTALL | re.VERBOSE)

    codehilite_lang = 'html'


    @classmethod
    def set_jinja2_environment(cls, environment):
        cls.jinja2_environment = environment

    @classmethod
    def reset_isolated_file_counter(cls):
        cls.isolated_file_counter = 0

    @classmethod
    def get_next_isolated_preview_filename(cls):
        counter = cls.isolated_file_counter
        cls.isolated_file_counter += 1
        return '{}.preview.html'.format(counter)

    def __init__(self, md, settingsobject):
        super(StyleExampleBlockPreprocessor, self).__init__(md)
        self.checked_for_codehilite = False
        self.settingsobject = settingsobject
        self.codehilite_conf = {
            'lang': self.codehilite_lang
        }

    def _render_jinja2_template(self, template_name, **render_kwargs):
        template = self.__class__.jinja2_environment.get_template(template_name)
        return template.render(**render_kwargs).strip()

    def run(self, lines):
        """ Match and store Example Code Blocks in the HtmlStash. """

        # Check for html hilite extension
        if not self.checked_for_codehilite:
            for ext in self.markdown.registeredExtensions:
                if isinstance(ext, codehilite.CodeHiliteExtension):
                    self.codehilite_conf = ext.config.copy()
                    # self.codehilite_conf['lang'] = self.codehilite_lang
                    # if 'force_linenos' in self.codehilite_conf:
                    #     # force_linenos is deprecated and does not work with
                    #     # the CodeHilite class. I guess it is converted to
                    #     # the ``linenums`` option in the extension.
                    #     del self.codehilite_conf['force_linenos']
                    # if 'pygments_style' in self.codehilite_conf:
                    #     del self.codehilite_conf['pygments_style']
                    break

            self.checked_for_codehilite = True

        text = "\n".join(lines)
        while 1:
            m = self.FENCED_BLOCK_RE.search(text)
            if m:
                html = m.group('html')
                examplestyle = m.group('examplestyle')
                if examplestyle is None:
                    examplestyle = 'default'
                previewfilename = self._save_htmlfile(html, examplestyle)
                rendered = self._render_examplehtml(html, examplestyle, previewfilename)
                placeholder = self.markdown.htmlStash.store(rendered, safe=True)
                text = '%s\n%s\n%s'% (text[:m.start()], placeholder, text[m.end():])
            else:
                break

        return text.split("\n")

    def _save_htmlfile(self, html, examplestyle):
        if examplestyle == 'isolated':
            previewfilesdir = os.path.join(self.settingsobject['outdir'],
                self.settingsobject['previewsdir'])
            if not os.path.exists(previewfilesdir):
                os.mkdir(previewfilesdir)

            unique_filename = self.__class__.get_next_isolated_preview_filename()
            with open(os.path.join(previewfilesdir, unique_filename), 'wb') as f:
                preview = self._render_jinja2_template('preview-standalone.html',
                    html=html)
                f.write(preview)
            return unique_filename
        else:
            return None

    def _render_examplehtml(self, html, examplestyle, previewfilename):
        codeblock = self._render_hilighted_codeblock(html)
        previewblock = self._render_previewblock(html, examplestyle, previewfilename)
        # return '<div>HEI</div>'
        return self._render_jinja2_template('example.html',
            examplestyle=examplestyle,
            codeblock=codeblock,
            previewblock=previewblock)

    def _render_hilighted_codeblock(self, html):
        highliter = codehilite.CodeHilite(html,
            lang=self.codehilite_lang,
            noclasses=self.codehilite_conf.get('noclasses', [True, None])[0]
        )
        return highliter.hilite()

    def _render_previewblock(self, html, examplestyle, previewfilename):
        return self._render_jinja2_template(
            'preview-{}.html'.format(examplestyle),
            html=html,
            examplestyle=examplestyle,
            previewfilename=previewfilename)

    def _escape(self, txt):
        """ basic html escaping """
        txt = txt.replace('&', '&amp;')
        txt = txt.replace('<', '&lt;')
        txt = txt.replace('>', '&gt;')
        txt = txt.replace('"', '&quot;')
        return txt




def to_html(input_markdown, settingsobject):
    codehilite_conf = {
        'linenums': False,
        'noclasses': True
    }
    md = markdown.Markdown(
        output_format='html5',
        safe_mode="escape",
        extensions=[
            'codehilite', # Syntax hilite code
            'fenced_code', # Support github style code blocks
            'nl2br', # Support github style newline handling
            'sane_lists', # Break into new ul/ol tag when the next line starts with another class of list indicator
            'smart_strong', # Do not let hello_world create an <em>,
            'def_list', # Support definition lists
            'tables', # Support tables
            StyleExampleExtension(configs={'settings': settingsobject}),
        ],
        extension_configs={
            'codehilite': codehilite_conf
        })
    return u'<div class="vitalstyles-reset vitalstyles-markdown">{}</div>'.format(md.convert(input_markdown))


import logging
import json
import os
import pprint

log = logging.getLogger(__name__)


class Settings(object):
    """
    Parses a JSON file for user defined settings.
    """
    defaults  = {
        # Path to your CSS file relative to the settings file (or CWD if no settings file).
        'preview_cssfile': None,

        # The output directory relative to the settings file (or CWD if no settings file).
        'outdir': 'vitalstyles_styleguide',

        # A list of files or directories to parse.
        # Directories is searched recursively for ``.scss``,
        # ``.sass``, ``.css`` and ``.less`` files.
        'inpaths': [os.getcwd()],

        # User defined templates directory.
        'template_dir': None,

        # The title of the guide
        'title': 'Vitalstyles style guide',

        # The subdirectory of ``outdir`` where we want to put isolated previews.
        'previewsdir': 'previews',
    }

    def __init__(self, filename=None):
        self.settings = self.defaults.copy()
        self.basedir = os.getcwd()
        if filename and os.path.exists(filename):
            log.debug('Loading settings from %s', filename)
            self.filename = filename
            self.basedir = os.path.dirname(filename)
            with open(self.filename, 'rb') as f:
                self.rawdata = f.read()
            usersettings = json.loads(self.rawdata)
            for key, value in usersettings.iteritems():
                if not key in self.defaults:
                    raise KeyError('Invalid settings key: {}.'.format(key))
                else:
                    self.settings[key] = value
        else:
            log.info('No settings file found. Using default settings:\n%s', pprint.pformat(self.settings))

    def __getitem__(self, key):
        return self.settings[key]

    def __contains__(self, key):
        return key in self.settings

    def get_outdir_path(self):
        return os.path.abspath(os.path.join(self.basedir, self.settings['outdir']))

    def get_preview_cssfilepath_relative_to_outdir(self):
        path = os.path.abspath(os.path.join(self.basedir, self.settings['preview_cssfile']))
        return os.path.relpath(path, self.get_outdir_path())

    def get_template_dir_path(self):
        if self.settings['template_dir']:
            return os.path.abspath(os.path.join(self.basedir, self.settings['template_dir']))
        else:
            return None


from setuptools import setup, find_packages


setup(
    name = 'vitalstyles',
    description = 'Generate CSS/SASS/LESS documentation with previews using Markdown in comments.',
    version = '1.0',
    license = 'BSD',
    author = 'Espen Angell Kristiansen',
    author_email = 'post@espenak.net',
    url = 'https://github.com/appressoas/vitalstyles',
    packages=find_packages(exclude=['manage']),
    install_requires = [
        'setuptools',
        'Markdown',
        'Jinja2',
        'pygments',
    ],
    classifiers = [
        'Intended Audience :: Developers',
        'License :: OSI Approved',
        'Programming Language :: Python'
    ],
    include_package_data=True,
    zip_safe=True,
    entry_points = {
        'console_scripts': [
          'vitalstyles-cli = vitalstyles.cli:cli',
        ],
    },
)



import argparse
import logging

from . import guide
from . import settings


log = logging.getLogger(__name__)


def cli():
    parser = argparse.ArgumentParser(description='Create a CSS/LESS/SASS style guide.')
    # parser.add_argument('fspaths', nargs='+',
        # help="Files or directories.")
    parser.add_argument('-f', '--settingsfile',
        dest='settingsfile', default='vitalstyles.json',
        help='Path to settings file. Defaults to "vitalstyles.json".')
    parser.add_argument('-l', '--loglevel',
        dest='loglevel', default='INFO',
        choices=['DEBUG', 'INFO', 'ERROR'], help='Loglevel.')
    args = parser.parse_args()
    logging.basicConfig(
        format='%(levelname)s: %(message)s',
        level=getattr(logging, args.loglevel)
    )
    settingsobject = settings.Settings(args.settingsfile)
    guide.Guide(settingsobject).render()


if __name__ == '__main__':
    cli()